--[[

         Titan Panel [Item Count]
         50200-1.03.2.b1

]]--

TitanItemCount = LibStub("AceAddon-3.0"):NewAddon("TitanItemCount", "AceConsole-3.0", 
   "AceComm-3.0", "AceEvent-3.0", "AceSerializer-3.0", "AceTimer-3.0", "AceHook-3.0");
TitanItemCount.version = "50200-1.03.2.b1";
TitanItemCount.author = "tfetherston";

local C_YELLOW = "|cffffff00"
local C_GREEN  = "|cff00ff00"
local C_WHITE  = "|cffffffff"
local C_RED    = "|cffff0000"
local C_TURQ   = "|cff22ee55"
local C_MGNTA  = "|cffff0088"
local C_PURPLE = "|cffEE22aa"

local menutext = "Titan Item Count "..C_TURQ.."(" .. 
   GetAddOnMetadata("TitanItemCount", "Version") .. ")";
local buttonlabel = "Item Count: "
local ID = "ItmC"

local elapQoH, QoH, prevQoH = 0, 0.0, -2
local GetUnitQoH = 0 
local startVal = 0
local GoalIsMet = false
local AlreadyAlerted = true  -- Don't alert on startup, only reset for alert when qty changes
local FirstRun = true
local AllowDebug = false

local AllianceBell = "Sound/Doodad/Belltollalliance.Ogg"
local HordeBell = "Sound/Doodad/Belltollhorde.Ogg"
local FactionBell = AllianceBell

--TitanItemCount_UpdateInterval = 1.0;  -- does this really do anything?


--[[
-- ------------------[ Settings for the Addon Options Panel ]---------------
]]--

local options = {
	name = "Item Count",
	handler = TitanItemCount,
	type = "group",
	args = {
		info1 = {
			type		= "description",
			name		= "Version " .. TitanItemCount.version,
			order		= 0,
		},
		info2 = {
			type		= "description",
			name		= "Authors: " .. TitanItemCount.author,
			order		= 1,
		},
		header = {
			type		= "header",
			name		= "General Options",
			order		= 2,
		},
		Item = {
			type     = 'input',
			name     = "Item to Count",
			desc     = "Inventory item to track quantity",
			get      = function() return TitanItemCount.db.char.Item end,
			set      = "SetItem",
		   validate = "ValidItem",
		   usage    = "Enter an item from your inventory - type, drag, or alt-click",
			width    = full,
			order    = 3,
		},
		space1 = {
		   type     = "description",
		   name     = "",
		   order    = 4,
		},
		ShowItem = {
		   type     = 'toggle',
		   name     = "Show Item Name",
		   desc     = "Show the item name in the Titan button?",
		   get      = function() return TitanGetVar(ID,"ShowItem") end,
		   set      = "SetShowItem",
		   width    = full,
		   order    = 5,
		},
		space2 = {
		   type     = "description",
		   name     = "",
		   order    = 6,
		},
		Goal = {
		   type     = 'input',
		   name     = "Goal Quantity",
		   desc     = "Zero to disable, positive integer for alert when qty reached",
		   get      = function() return tostring(TitanItemCount.db.char.Goal) end,
		   set      = "SetGoal",
		   validate = "ValidGoal",
		   usage    = "Enter an integer zero or higher",
		   width    = full,
		   order    = 8,
		},
		info3 = {
			type		= "description",
			name		= "Set Goal Quantity to zero if you don't want to hear about it.",
			order		= 9,
		},	
	}
};


--[[
-- ------------------[ Saved Variable Data Table ]---------------
]]--

local defaults = {
	char = {
		Item = "Elixir of Water Walking",
		ShowItem = true,
		Goal = 0,
		debug = false,
	}
};


--[[
-- ------------------[ Addon Object Functions ]---------------
]]--

function TitanItemCount:OnInitialize()
	-- Load Options DB
	self.db = LibStub("AceDB-3.0"):New("TitanItemCountDB", defaults);

	-- Initialize Config Panels
	self:InitConfig();

   -- Register Events

	-- Delay welcome message.
	self:ScheduleTimer(function() 
			self:Print("version (" .. self.version .. ") loaded.")
			self:Print("Currently tracking " .. TitanItemCount.db.char.Item)
		end, 4);

   -- Set bell sound according to player faction
   FactionBell = AllianceBell
   if UnitFactionGroup("player") == "Horde" then
      FactionBell = HordeBell
   end

end

function TitanItemCount:OnEnable()
	self:Hook("ContainerFrameItemButton_OnModifiedClick", true)
end


function TitanItemCount:InitConfig()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Item Count", options, "tic");
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Item Count");
end


function TitanItemCount:ticAnnounce()
   self:Print(C_YELLOW .. "now counting " .. TitanItemCount.db.char.Item .. ".")
end


function TitanItemCount:qtyAnnounce()
   self:Print(C_YELLOW .. "Goal Quantity is now " .. C_WHITE .. TitanItemCount.db.char.Goal .. ".")
end


-- AltClick Handling
function TitanItemCount:ContainerFrameItemButton_OnModifiedClick(...)
   if select(2, ...) == "LeftButton" and IsAltKeyDown() and 
      not IsControlKeyDown() and not IsShiftKeyDown() and 
      not CursorHasItem() then

		bagID, slot = (...):GetParent():GetID(), (...):GetID()
      texture, itemCount, locked, quality, readable, lootable, itemLink = 
         GetContainerItemInfo(bagID, slot);
      -- no need to validate, we know it's in the bags
      TitanItemCount.db.char.Item = itemLink
      TitanItemCount:ticAnnounce()
      startVal = 0

   end
end


function TitanItemCount:DoAlert()
   PlaySoundFile(FactionBell, "Master")
   CombatText_AddMessage("Item Count Goal Attainied", CombatText_StandardScroll, 
      0.9, 0.2, 0.5, nil, true, nil);
end



--[[    Interface with AceConfig Options Panel   ]]--

-- assumes this is only run when the item changes...
function TitanItemCount:SetItem(info, input)
   TitanItemCount.db.char.Item = input
   QoH = GetItemCount(TitanItemCount.db.char.Item)
   GoalIsMet = (QoH >= TitanItemCount.db.char.Goal)
   -- when the tracked item is changed, and GoalIsMet, don't Alert
   -- but do set up to alert if the qoh is lower than goal
   AlreadyAlerted = GoalIsMet
   prevQoH = 0
   TitanPanelButton_UpdateButton(ID)
   TitanItemCount:ticAnnounce()
end


-- assumes this is only run when the goal changes...
function TitanItemCount:SetGoal(info, input)
   TitanItemCount.db.char.Goal = tonumber(input)
   GoalIsMet = (QoH >= TitanItemCount.db.char.Goal)
   -- when the goal is changed, and GoalIsMet, don't Alert:
   -- assume the user knows s/he is setting the Goal
   -- higher than current QoH
   AlreadyAlerted = GoalIsMet
   prevQoH = 0
   TitanPanelButton_UpdateButton(ID)
   TitanItemCount:qtyAnnounce()
end


function TitanItemCount:SetShowItem(info, input)
   TitanSetVar(ID, "ShowItem", input);
   TitanItemCount.db.char.ShowItem = input;
   TitanPanelButton_UpdateButton(ID)
end


function TitanItemCount:ValidItem(info, input)
   local tcount
   tcount = GetItemCount(input)
   if tcount < 1 then 
      self:Print("Tracked item must be something in your bags.")
      return false
   end
   return true
end


function TitanItemCount:ValidGoal(info, input)
   local tnum
   tnum = tonumber(input)
   if not tnum or tnum < 0 then 
      self:Print("You must enter an integer zero or higher")
      return false
   end
   return true
end


--[[
-- ------------------[ Titan Panel Button Event handlers ]---------------
]]--

function TitanPanelItmCButton_OnLoad(self)
   if not TitanPanelButton_UpdateButton then
      return;
   end

	self.registry = {
		id = ID,
		menuText = menutext,
		buttonTextFunction = "TitanPanelItmCButton_GetButtonText",
		tooltipTitle = GREEN_FONT_COLOR_CODE .. "Item Count",
		tooltipTextFunction = "TitanPanelItmCButton_GetTooltipText",
		frequency = 2,
		icon = "Interface\\AddOns\\TitanBag\\TitanBag",
		iconWidth = 16,
		category = "Information",
		savedVariables = {
			ShowIcon = 1,
			ShowLabelText = false,
			ShowItem = false,
			debug = false,
		},
	}

end


function TitanPanelItmCButton_OnUpdate(this, elapsed)

   if elapsed == nil then elapsed = 2; end
   elapQoH = elapQoH + elapsed;
   if elapQoH < 1 then return end

   QoH = GetItemCount(TitanItemCount.db.char.Item)

   -- only update the button if qty has changed
   if QoH == prevQoH then return end
   prevQoH = QoH
   GoalIsMet = (QoH >= TitanItemCount.db.char.Goal)
   TitanPanelButton_UpdateButton(ID)
   elapQoH = 0;
   
   FirstRun = false

end


function TitanPanelItmCButton_GetButtonText()
	local QoHtext, countcolor, alertcolor
	countcolor = C_WHITE  -- default white
	alertcolor = C_MGNTA  -- alert redviolet?
	QoHtext = "-"
	if not QoH then
	   QoH = 0
	end
	if TitanItemCount.db.char.Goal > 0 and GoalIsMet then
	   countcolor = alertcolor
	   if not FirstRun and not AlreadyAlerted then 
	      TitanItemCount:DoAlert()
	   end
	   AlreadyAlerted = true
	   --FirstRun = false
	end
   QoHtext = countcolor .. string.format(" %.0f ", QoH) .. "|r"
	if TitanGetVar(ID, "ShowItem") then
	   QoHtext = QoHtext .. TitanItemCount.db.char.Item
	end
	return buttonlabel, QoHtext
end


function TitanPanelItmCButton_GetTooltipText()
   local ttString
   ttString = C_YELLOW .. "Item: \t" .. C_WHITE .. TitanItemCount.db.char.Item .. "|r" ..
      "\n" .. C_YELLOW .."Qty on hand:\t" .. C_WHITE .. string.format("%.0f", QoH) .. "|r"
   if TitanItemCount.db.char.Goal > 0 then
	   ttString = ttString .. 
	      "\n"..C_YELLOW.."Goal quantity:\t" ..C_WHITE .. tostring(TitanItemCount.db.char.Goal) .. "|r"
      if GoalIsMet then
         ttString = ttString .. "\n"..C_PURPLE.."GOAL QUANTITY ACHIEVED|r"
      end
	end
   ttString = ttString .. "\n"..C_GREEN.."Hint: left-click to open Options Panel,\n"..
      C_GREEN.."or use the "..C_WHITE.."/tic"..C_GREEN.." slash command."
   if AllowDebug and TitanGetVar(ID,"debug") then
      ttString = ttString .."\n"
      ttString = ttString .."\n"..C_YELLOW.."First Run:\t"..tostring(FirstRun)
      ttString = ttString .."\n"..C_YELLOW.."GoalIsMet:\t"..tostring(GoalIsMet)
      ttString = ttString .."\n"..C_YELLOW.."AlreadyAlerted:\t"..tostring(AlreadyAlerted)
      ttString = ttString .."\n"..C_YELLOW.."prevQoH:\t"..tostring(prevQoH)
   end
   return ttString
end


function TitanPanelItmCButton_OnShow(self)
	TitanPanelButton_OnShow(self);
end


function TitanPanelItmCButton_OnHide()
end


function TitanPanelItmCButton_OnClick(self, button)
   if (button == "LeftButton") then
      InterfaceOptionsFrame_OpenToCategory("Item Count")
   end
end


--[[
-- ------------------[ Titan Panel Functions ]---------------
]]--

local temp = {}
local function UIDDM_Add(text, func, checked, keepShown)
	temp.text, temp.func, temp.checked, temp.keepShownOnClick = text, 
	   func, checked, keepShown;
	UIDropDownMenu_AddButton(temp)
end

function TitanPanelRightClickMenu_PrepareItmCMenu()
	TitanPanelRightClickMenu_AddTitle(TitanPlugins[ID].menuText)	
	TitanPanelRightClickMenu_AddToggleIcon(ID)
	TitanPanelRightClickMenu_AddToggleLabelText(ID)
	TitanPanelRightClickMenu_AddToggleVar("Show Item Name", ID, "ShowItem", nil, level)
	TitanPanelRightClickMenu_AddSpacer()
	if AllowDebug == true then
	   TitanPanelRightClickMenu_AddToggleVar("Debug", ID, "debug", nil, level)
	end
	TitanPanelRightClickMenu_AddCommand("Hide", ID, TITAN_PANEL_MENU_FUNC_HIDE, level)
end
